"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _types = require("@babel/types");

var _utils = require("./utils");

var DefaultRefLabel = "ref";
var DefaultRefFactory = "React.useState";

function reactLabelSugar(_, options) {
  var _options$RefFactory = options.RefFactory,
      RefFactory = _options$RefFactory === void 0 ? DefaultRefFactory : _options$RefFactory,
      _options$RefLabel = options.RefLabel,
      RefLabel = _options$RefLabel === void 0 ? DefaultRefLabel : _options$RefLabel;
  return {
    visitor: {
      LabeledStatement: function LabeledStatement(path, state) {
        var node = path.node;
        var label = node.label,
            body = node.body;
        if (label.name !== RefLabel) return;

        if (!(0, _types.isExpressionStatement)(body)) {
          throw new Error("ref sugar must be an expression statement");
        }

        var expression = body.expression;

        if (!(0, _types.isAssignmentExpression)(expression)) {
          throw new Error("ref sugar expression must be an assignment");
        }

        var left = expression.left,
            right = expression.right;

        if (!(0, _types.isIdentifier)(left)) {
          throw new Error("ref sugar assignment left must be an identifier");
        }

        var refItem = {
          identify: left,
          modifier: path.scope.generateUidIdentifier("set".concat((0, _utils.capitalize)(left.name)))
        };
        path.replaceWith((0, _types.variableDeclaration)("const", [(0, _types.variableDeclarator)((0, _types.arrayPattern)([refItem.identify, refItem.modifier]), (0, _types.callExpression)((0, _types.identifier)(RefFactory), [right]))]));
        if (!state.reactRefs) state.reactRefs = [];
        state.reactRefs.push(refItem);
      },
      AssignmentExpression: function AssignmentExpression(path, state) {
        if (!state.reactRefs) return;
        var node = path.node;
        if (!(0, _types.isIdentifier)(node.left)) return;
        var identify = node.left.name;
        var ref = state.reactRefs.find(function (ref) {
          return ref.identify.name === identify;
        });
        if (!ref) return;

        if (node.operator === "=") {
          path.replaceWith((0, _types.callExpression)(ref.modifier, [node.right]));
        } else {
          path.replaceWith((0, _types.callExpression)(ref.modifier, [(0, _types.arrowFunctionExpression)([ref.identify], (0, _types.binaryExpression)(node.operator.slice(0, -1), ref.identify, node.right))]));
        }
      },
      UpdateExpression: function UpdateExpression(path, state) {
        if (!state.reactRefs) return;
        var node = path.node;
        if (!(0, _types.isIdentifier)(node.argument)) return;
        var identify = node.argument.name;
        var ref = state.reactRefs.find(function (ref) {
          return ref.identify.name === identify;
        });
        if (!ref) return;
        path.replaceWith((0, _types.callExpression)(ref.modifier, [(0, _types.arrowFunctionExpression)([ref.identify], (0, _types.binaryExpression)(node.operator === "++" ? "+" : "-", ref.identify, (0, _types.numericLiteral)(1)))]));
      }
    }
  };
}

var _default = reactLabelSugar;
exports["default"] = _default;